package cn.fw.valhalla.controller.task;

import cn.fw.valhalla.component.ClueChangeProducer;
import cn.fw.valhalla.domain.db.follow.FollowClue;
import cn.fw.valhalla.domain.enums.FollowTypeEnum;
import cn.fw.valhalla.sdk.enums.CustomerFollowTypeEnum;
import cn.fw.valhalla.sdk.result.ClueChangeResult;
import cn.fw.valhalla.service.data.FollowClueService;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.Objects;
import java.util.concurrent.CompletableFuture;

/**
 * 线索变更后的通知事件
 *
 * @author : kurisu
 * @version : 1.0
 * @className : ClueChangeNoticeTask
 * @description : 线索变更后的通知事件
 * @date : 2022-12-07 11:45
 */
@Component
@Slf4j
@ConditionalOnProperty(prefix = "task", name = "switch", havingValue = "on")
public class ClueChangeNoticeTask {
    private final ClueChangeProducer clueChangeProducer;
    private final FollowClueService followClueService;
    private final StringRedisTemplate redisTemplate;
    @Value("${spring.cache.custom.global-prefix}:follow:clue:change")
    @Getter
    private String clueChangeKeyPrefix;

    @Autowired
    public ClueChangeNoticeTask(final ClueChangeProducer clueChangeProducer,
                                final FollowClueService followClueService,
                                final StringRedisTemplate redisTemplate) {
        this.clueChangeProducer = clueChangeProducer;
        this.followClueService = followClueService;
        this.redisTemplate = redisTemplate;
    }

    @Scheduled(initialDelay = 1000 * 30, fixedRate = 1000 * 60)
    public void sendStopNotice() {
        String pop = null;
        while ((pop = redisTemplate.opsForSet().pop(generateStopKey())) != null) {
            try {
                final Long clueId = Long.valueOf(pop);
                CompletableFuture.runAsync(() -> {
                    FollowClue clue = followClueService.getById(clueId);
                    if (Objects.nonNull(clue)) {
                        ClueChangeResult result = new ClueChangeResult();
                        result.setFrameNo(clue.getVin());
                        result.setGroupId(clue.getGroupId());
                        result.setChangeType(ClueChangeResult.ChangeType.STOP);
                        if (FollowTypeEnum.RM.equals(clue.getClueType())) {
                            result.setClueType(CustomerFollowTypeEnum.RM.getValue());
                        } else if (FollowTypeEnum.IR.equals(clue.getClueType())) {
                            result.setClueType(CustomerFollowTypeEnum.IR.getValue());
                        }
                        clueChangeProducer.send(result);
                    }
                });
            } catch (Exception ex) {
                log.error(ex.getMessage(), ex);
            }
        }
    }

    private String generateAddKey() {
        return String.format("%s:%s", getClueChangeKeyPrefix(), "ADD");
    }

    private String generateStopKey() {
        return String.format("%s:%s", getClueChangeKeyPrefix(), "STOP");
    }
}
